package com.test.referencedata.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

import org.apache.lucene.analysis.KeywordAnalyzer;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanClause.Occur;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.util.Version;
import org.infinispan.Cache;
import org.infinispan.config.Configuration;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.query.CacheQuery;
import org.infinispan.query.QueryFactory;
import org.infinispan.query.backend.QueryHelper;

import com.test.referencedata.DataStore;

public class DataStoreImpl<K, V> implements DataStore<K, V> {

	private Cache<K, V> cache;
	private QueryHelper qh;

	public DataStoreImpl(String dataStoreName, String configFileName,
			Class<V> vClazz) throws IOException {
		EmbeddedCacheManager cacheManager = new DefaultCacheManager(
				configFileName);
		this.cache = cacheManager.getCache(dataStoreName);
		Properties properties = new Properties();
		properties.setProperty("hibernate.search.default.directory_provider",
				"org.hibernate.search.store.RAMDirectoryProvider");
		qh = new QueryHelper(cache, properties, vClazz);
	}

	@Override
	public void put(K key, V value) {
		cache.put(key, value);
	}

	@Override
	public void putAll(Map<K, V> entries) {
		cache.startBatch();
		cache.putAll(entries);
		cache.endBatch(true);
	}

	@Override
	public V get(K key) {
		return cache.get(key);
	}

	@Override
	public Collection<V> getAll(Collection<K> keys) {
		Collection<V> values = new ArrayList<V>();
		for (K key : keys) {
			values.add(get(key));
		}
		return values;
	}

	@Override
	public Collection<V> getAll() {
		return cache.values();
	}

	@SuppressWarnings("unchecked")
	@Override
	public Collection<V> find(Map<Object, Object> searchParams) {
		QueryFactory qf = new QueryFactory(cache, qh);
		BooleanQuery mainQuery = new BooleanQuery();
		for (Entry<Object, Object> entry : searchParams.entrySet()) {
			Query query = new TermQuery(new Term(entry.getKey().toString(),
					entry.getValue().toString()));
			mainQuery.add(query, Occur.MUST);
		}

		CacheQuery cq = qf.getQuery(mainQuery);
		return (Collection<V>) cq.list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public Collection<V> find(String searchQuery) {
		Query query;
		try {
			query = new QueryParser(Version.LUCENE_29, searchQuery,
					new KeywordAnalyzer()).parse(searchQuery);
			QueryFactory qf = new QueryFactory(cache, qh);
			CacheQuery cq = qf.getQuery(query);
			Collection<V> results = new HashSet<V>();
			for (Object result : cq) {
				results.add((V) result);
			}
			return results;
		} catch (ParseException e) {
			e.printStackTrace();
			return null;
		}
	}

}
